@@ -83,6 +83,27 @@ impl Client {
         }
     }
     pub fn login(&mut self, password: String) {
+        let should_req_well_known = match self.host.split(":").last().unwrap().parse::<usize>() {
+            Ok(port) => {
+                if port != 443 {
+                    false
+                } else {
+                    true
+                }
+            },
+            Err(_) => {
+                true
+            }
+        };
+        if should_req_well_known {
+            if let Ok(z) = ureq::get(&self.host.clone()).call() {
+                let resp: Value = z.into_json().unwrap();
+                let base_url: String = resp["m.homeserver"]["base_url"].as_str().unwrap().to_string();
+                self.host = base_url.trim_end_matches("/").to_string();
+                //self.me = UserId::try_from(format!("@{}:{}", self.me.localpart(), self.host.clone().split_once("://").unwrap().1).as_str()).unwrap();
+            }
+        }
+        
         let content = json!({
             "type": "m.login.password",
             "identifier": {
@@ -93,7 +114,8 @@ impl Client {
         });
         let val: Value = self.post_r0("login", content, None).unwrap().into_json().unwrap();
         //println!("{:#}", val);
-        self.data.auth_token = val["access_token"].as_str().unwrap().to_string()
+        self.data.auth_token = val["access_token"].as_str().unwrap().to_string();
+        self.me = UserId::try_from(val["user_id"].as_str().unwrap()).unwrap();
     }
     pub fn get_me(&self) -> UserId {
         self.me.clone()
@@ -110,7 +132,7 @@ impl Client {
         //x
         
         let mut resp = r.clone().send_json(content.clone()).or_any_status();
-        while let Err(e) = resp {
+        while let Err(_e) = resp {
             resp = r.clone().send_json(content.clone()).or_any_status();
             //eprintln!("{}", e);
         }
@@ -129,7 +151,7 @@ impl Client {
         //x
         
         let mut resp = r.clone().send_json(content.clone()).or_any_status();
-        while let Err(e) = resp {
+        while let Err(_e) = resp {
             resp = r.clone().send_json(content.clone()).or_any_status();
             //eprintln!("{}", e);
         }
@@ -144,7 +166,7 @@ impl Client {
         }
 
         let mut resp = r.clone().send_json(content.clone()).or_any_status();
-        while let Err(e) = resp {
+        while let Err(_e) = resp {
             resp = r.clone().send_json(content.clone()).or_any_status();
             //eprintln!("{}", e);
         }
@@ -159,7 +181,7 @@ impl Client {
         }
         let mut resp = r.clone().call().or_any_status();
         
-        while let Err(e) = resp {
+        while let Err(_e) = resp {
             resp = r.clone().call().or_any_status();
             //eprintln!("{}", e);
         }
@@ -208,6 +230,7 @@ impl Client {
         let this_room_id_val: Value = self.get(format!("directory/room/{}", urlencoding::encode(room_alias.as_str()).to_string().as_str()), None).unwrap().into_json().unwrap();
         ruma::RoomId::try_from(this_room_id_val["room_id"].as_str().unwrap()).unwrap()
     }
+    /*
     #[cfg(not(target_os = "android"))]
     pub fn start(&mut self, password: impl Into<String>, room_ids: Vec<impl Into<String>>) {
         let pass = password.into();
@@ -217,11 +240,80 @@ impl Client {
             let room_id = ruma::RoomId::try_from(this_room_id_val["room_id"].as_str().unwrap()).unwrap();
         }
     }
+    */
     pub fn send_message(&self, room_id: RoomId, message: impl Into<String>) {
         let msg = message.into();
         let txid = Uuid::new_v4().to_simple().encode_lower(&mut Uuid::encode_buffer()).to_string();
         self.put(format!("rooms/{}/send/m.room.message/{}", room_id.clone(), txid).as_str(), json!({"body": msg, "msgtype": "m.text"}), None).unwrap();
     }
+
+
+    pub fn extract_messages(&self, sync_results: EventResponse) -> Vec<MessageInfo> {
+        let mut messages: Vec<MessageInfo> = Vec::new();
+        //if let RoomsmTypeData::Joined(ref x) = event_args.room_data {
+        let x = sync_results.rooms.join;
+        for (room_id, joined_room) in x.iter() {
+            //if event_args.ctrlc_handler.should_continue() {
+            /*
+            instructions.push(Instructions::AddRoom(room_id.clone()));
+            for event in joined_room.state.events.clone() {
+                if let ruma::events::AnySyncStateEvent::RoomAliases(room_aliases_event) = event.deserialize().unwrap() {
+                    let aliases = room_aliases_event.content.aliases;
+                    for alias in aliases {
+                        instructions.push(Instructions::SaveRoomAlias(room_id.clone(), alias));
+                    }
+                } //else if let ruma::events::AnySyncStateEvent::RoomCanonicalAlias
+            }
+            */
+            //if !event_args.starting {
+            for message_info in joined_room.timeline.events.iter().filter_map(|m| {
+                if let ruma::events::AnySyncRoomEvent::Message(message_event) = m.deserialize_as().unwrap() {
+                    if let ruma::events::AnySyncMessageEvent::RoomMessage(room_message) = message_event {
+                        let sender = room_message.sender;
+                        
+                        if sender == self.me {
+                            return None;
+                        }
+                        
+                        if let room::message::MessageType::Text(text_message_event_content) = room_message.content.msgtype {
+                            let message = text_message_event_content.body;
+                            //event_args.tx.send(format!("<{}> {}", sender, message)).unwrap();
+                            //if message.starts_with(&event_args.prefix) {
+                                /*
+                                let words: Vec<String> = message.split_whitespace().map(|w| w.to_string()).collect();
+                                let args = words[1..].to_vec();
+                                let mut room_aliases: Vec<RoomAliasId> = Vec::new();
+                                if event_args.room_to_aliases.contains_key(&room_id) {
+                                    room_aliases.append(&mut event_args.room_to_aliases[&room_id].clone());
+                                }
+                                */
+                            return Some(MessageInfo{
+                                message: message.clone(),
+                                //words: words,
+                                //args: args,
+                                sender: sender,
+                                room_id: room_id.clone(),
+                                //room_aliases: room_aliases,
+                            });
+                            //}
+                        }
+                    }
+                }
+                None
+            }) {
+                messages.push(message_info);
+            }
+            //}
+            //} //else {
+                //instructions.push(Instructions::Quit(event_args.cleanup_on_ctrlc));
+                //return instructions;
+            //}
+        }
+        //}
+        messages
+    }
+
+
 }
 
 pub struct MessageInfo {
@@ -230,65 +322,3 @@ pub struct MessageInfo {
     pub message: String,
 }
 
-pub fn extract_messages(sync_results: EventResponse, me: UserId) -> Vec<MessageInfo> {
-    let mut messages: Vec<MessageInfo> = Vec::new();
-    //if let RoomsmTypeData::Joined(ref x) = event_args.room_data {
-    let mut x = sync_results.rooms.join;
-    for (room_id, joined_room) in x.iter() {
-        //if event_args.ctrlc_handler.should_continue() {
-        /*
-        instructions.push(Instructions::AddRoom(room_id.clone()));
-        for event in joined_room.state.events.clone() {
-            if let ruma::events::AnySyncStateEvent::RoomAliases(room_aliases_event) = event.deserialize().unwrap() {
-                let aliases = room_aliases_event.content.aliases;
-                for alias in aliases {
-                    instructions.push(Instructions::SaveRoomAlias(room_id.clone(), alias));
-                }
-            } //else if let ruma::events::AnySyncStateEvent::RoomCanonicalAlias
-        }
-        */
-        //if !event_args.starting {
-        for message_info in joined_room.timeline.events.iter().filter_map(|m| {
-            if let ruma::events::AnySyncRoomEvent::Message(message_event) = m.deserialize_as().unwrap() {
-                if let ruma::events::AnySyncMessageEvent::RoomMessage(room_message) = message_event {
-                    let sender = room_message.sender;
-                    if sender == me {
-                        return None;
-                    }
-                    if let room::message::MessageType::Text(text_message_event_content) = room_message.content.msgtype {
-                        let message = text_message_event_content.body;
-                        //event_args.tx.send(format!("<{}> {}", sender, message)).unwrap();
-                        //if message.starts_with(&event_args.prefix) {
-                            /*
-                            let words: Vec<String> = message.split_whitespace().map(|w| w.to_string()).collect();
-                            let args = words[1..].to_vec();
-                            let mut room_aliases: Vec<RoomAliasId> = Vec::new();
-                            if event_args.room_to_aliases.contains_key(&room_id) {
-                                room_aliases.append(&mut event_args.room_to_aliases[&room_id].clone());
-                            }
-                            */
-                        return Some(MessageInfo{
-                            message: message.clone(),
-                            //words: words,
-                            //args: args,
-                            sender: sender,
-                            room_id: room_id.clone(),
-                            //room_aliases: room_aliases,
-                        });
-                        //}
-                    }
-                }
-            }
-            None
-        }) {
-            messages.push(message_info);
-        }
-        //}
-        //} //else {
-            //instructions.push(Instructions::Quit(event_args.cleanup_on_ctrlc));
-            //return instructions;
-        //}
-    }
-    //}
-    messages
-}
